//
//	MBsysTran - Release 8.1
//
//	Copyright 
//	Universite catholique de Louvain (UCLouvain) 
//	Mechatronic, Electrical Energy, and Dynamic systems (MEED Division) 
//	2, Place du Levant
//	1348 Louvain-la-Neuve 
//	Belgium 
//
//	http://www.robotran.be 
//
//	==> Generation Date: Tue Jan 30 10:32:55 2024
//
//	==> Project name: pendulum_spring_c
//
//	==> Number of joints: 4
//
//	==> Function: F6 - Sensors Kinematics
//
//	==> Git hash: 9034ed7a9ff7b13e6df2e2fd994ccfc949192e56
//
//	==> Input XML
//

#include <math.h> 

#include "mbs_data.h"
#include "mbs_sensor.h"

void mbs_gensensor(MbsSensor *sens,
MbsData *s, int isens)
{
#include "mbs_gensensor_pendulum_spring_c.h"

double *q, *qd, *qdd;
double **dpt;

q = s->q;
qd = s->qd;
qdd = s->qdd;

dpt = s->dpt;
 
// Trigonometric functions

S1 = sin(q[1]);
C1 = cos(q[1]);
S3 = sin(q[3]);
C3 = cos(q[3]);
S4 = sin(q[4]);
C4 = cos(q[4]);
 
// Augmented Joint Position Vectors

Dz23 = q[2]+dpt[3][3];
 
// Augmented Joint Position Vectors

 
// Sensor Kinematics


switch(isens)
{
case 1:

sens->P[1] = 0;
sens->P[2] = 0;
sens->P[3] = 0;
sens->R[1][1] = C1;
sens->R[1][3] = -S1;
sens->R[2][2] = (1.0);
sens->R[3][1] = S1;
sens->R[3][3] = C1;
sens->V[1] = 0;
sens->V[2] = 0;
sens->V[3] = 0;
sens->OM[1] = 0;
sens->OM[2] = qd[1];
sens->OM[3] = 0;
sens->J[5][1] = (1.0);
sens->A[1] = 0;
sens->A[2] = 0;
sens->A[3] = 0;
sens->OMP[1] = 0;
sens->OMP[2] = qdd[1];
sens->OMP[3] = 0;

break;

case 2:

RLcp2_12 = Dz23*S1;
RLcp2_32 = Dz23*C1;
ORcp2_12 = RLcp2_32*qd[1];
ORcp2_32 = -RLcp2_12*qd[1];
VIcp2_12 = ORcp2_12+qd[2]*S1;
VIcp2_32 = ORcp2_32+qd[2]*C1;
ACcp2_12 = ORcp2_32*qd[1]+RLcp2_32*qdd[1]+qdd[2]*S1+(2.0)*qd[1]*qd[2]*C1;
ACcp2_32 = -ORcp2_12*qd[1]-RLcp2_12*qdd[1]+qdd[2]*C1-(2.0)*qd[1]*qd[2]*S1;
sens->P[1] = RLcp2_12;
sens->P[2] = 0;
sens->P[3] = RLcp2_32;
sens->R[1][1] = C1;
sens->R[1][3] = -S1;
sens->R[2][2] = (1.0);
sens->R[3][1] = S1;
sens->R[3][3] = C1;
sens->V[1] = VIcp2_12;
sens->V[2] = 0;
sens->V[3] = VIcp2_32;
sens->OM[1] = 0;
sens->OM[2] = qd[1];
sens->OM[3] = 0;
sens->J[1][1] = RLcp2_32;
sens->J[1][2] = S1;
sens->J[3][1] = -RLcp2_12;
sens->J[3][2] = C1;
sens->J[5][1] = (1.0);
sens->A[1] = ACcp2_12;
sens->A[2] = 0;
sens->A[3] = ACcp2_32;
sens->OMP[1] = 0;
sens->OMP[2] = qdd[1];
sens->OMP[3] = 0;

break;

case 3:

sens->P[1] = dpt[1][2];
sens->P[2] = 0;
sens->P[3] = 0;
sens->R[1][1] = C3;
sens->R[1][3] = -S3;
sens->R[2][2] = (1.0);
sens->R[3][1] = S3;
sens->R[3][3] = C3;
sens->V[1] = 0;
sens->V[2] = 0;
sens->V[3] = 0;
sens->OM[1] = 0;
sens->OM[2] = qd[3];
sens->OM[3] = 0;
sens->J[5][3] = (1.0);
sens->A[1] = 0;
sens->A[2] = 0;
sens->A[3] = 0;
sens->OMP[1] = 0;
sens->OMP[2] = qdd[3];
sens->OMP[3] = 0;

break;

case 4:

ROcp4_14 = C3*C4-S3*S4;
ROcp4_34 = -C3*S4-S3*C4;
ROcp4_74 = C3*S4+S3*C4;
ROcp4_94 = C3*C4-S3*S4;
RLcp4_12 = dpt[3][6]*S3;
RLcp4_32 = dpt[3][6]*C3;
POcp4_12 = RLcp4_12+dpt[1][2];
OMcp4_22 = qd[3]+qd[4];
ORcp4_12 = RLcp4_32*qd[3];
ORcp4_32 = -RLcp4_12*qd[3];
OPcp4_22 = qdd[3]+qdd[4];
ACcp4_12 = ORcp4_32*qd[3]+RLcp4_32*qdd[3];
ACcp4_32 = -ORcp4_12*qd[3]-RLcp4_12*qdd[3];
sens->P[1] = POcp4_12;
sens->P[2] = 0;
sens->P[3] = RLcp4_32;
sens->R[1][1] = ROcp4_14;
sens->R[1][3] = ROcp4_34;
sens->R[2][2] = (1.0);
sens->R[3][1] = ROcp4_74;
sens->R[3][3] = ROcp4_94;
sens->V[1] = ORcp4_12;
sens->V[2] = 0;
sens->V[3] = ORcp4_32;
sens->OM[1] = 0;
sens->OM[2] = OMcp4_22;
sens->OM[3] = 0;
sens->J[1][3] = RLcp4_32;
sens->J[3][3] = -RLcp4_12;
sens->J[5][3] = (1.0);
sens->J[5][4] = (1.0);
sens->A[1] = ACcp4_12;
sens->A[2] = 0;
sens->A[3] = ACcp4_32;
sens->OMP[1] = 0;
sens->OMP[2] = OPcp4_22;
sens->OMP[3] = 0;

break;

default:

break;

}


// Number of continuation lines = 0

}
